#!/usr/bin/env python3

import sys
import time
import random
import math


# Path to the FIFO device of the emulated usb-serial
SERIAL_FIFO = "/tmp/si-tema2-serial0"

# syntax: (location, sensor_name, value_range)
SENSORS = [
    ("ext", "humidity", [20, 80, 2, 1]),
    ("ext", "temp", [0, 25, 1, 2]),
    ("ext", "rain", [0, 1, 1, 1]),
    ("int", "humidity", [30, 70, 1, 1]),
    ("int", "temp", [20, 35, 0.8, 2]),
]
SENSOR_IDS = [(d[0], d[1]) for d in SENSORS]
# Simulation step delay (seconds)
STEP_DELAY = 0.5
# Simulate rain / random broken sensor
RAIN_START_TIME = 5
BROKEN_SENSOR_TIME = 8


def print_serial_line(ser, data):
    timestamp, location, name, value, flags = data
    line = f"{timestamp}; {location}; {name}; "
    if name == "temp":
        line += f"{value:.1f}"
    else:
        line += f"{value}"
    if flags is not None:
        line += f"; {flags}"
    ser.write(line + "\n")
    ser.flush()


def sensor_simulator(ser):
    step = 0
    broken_sensor = None
    old_values = {}
    try:
        while True:
            for location, name, value_intv in SENSORS:
                value = 0
                flags = None
                id = (location, name)
                if step >= BROKEN_SENSOR_TIME and not broken_sensor:
                    broken_sensor = random.choices(SENSOR_IDS, k=1)[0]
                    print("ERROR for ", broken_sensor)
                if id == broken_sensor:
                    value = 0
                    flags = "ERROR"
                elif name == "rain":
                    value = 1 if step >= RAIN_START_TIME else 0
                else:
                    if id not in old_values:
                        value = random.randint(value_intv[0], value_intv[1])
                    else:
                        value = old_values[id]
                        # fetch binomial distribution based on old value
                        value = random.normalvariate(value, value_intv[2])
                        if value > value_intv[1]:
                            value = value_intv[1] - 1
                        if value < value_intv[0]:
                            value = value_intv[0]
                        # round it
                        value = round(value * value_intv[3]) / value_intv[3]

                old_values[id] = value
                timestamp = math.floor(time.time() + step)
                data = (timestamp, location, name, value, flags)
                print_serial_line(ser, data)
                time.sleep(STEP_DELAY)
            step += 1

    except KeyboardInterrupt:
        print('Interrupted!')
        sys.exit(0)

if __name__ == "__main__":
    serial_fifo = SERIAL_FIFO
    if len(sys.argv) > 1:
        serial_fifo = sys.argv[1]
    with open(serial_fifo, "w") as ser:
        sensor_simulator(ser)

